<?php
// $Horde: turba/lib/Driver/ldap.php,v 1.11.2.1 2001/11/19 06:00:51 chuck Exp $

/**
 * Turba directory driver implementation for PHP's LDAP extension.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.11.2.1 $
 * @since   Turba 0.0.1
 * @package turba
 */
class Turba_Driver_ldap extends Turba_Driver {

    /** String containing the hostname of the LDAP server.  */
    var $server = 'localhost';
    
    /** The port that the ldap server is running on. */
    var $port = 389;
    
    /** String containing the LDAP search root. */
    var $root = '';
    
    /** Handle for the current LDAP connection. */
    var $ds = 0;

    /** String containing the character set encoding. */
    var $encoding = '';
    
    
    /**
     * Constructs a new Turba LDAP driver object.
     *
     * @param $params       Hash containing additional configuration parameters.
     */
    function Turba_Driver_ldap($params)
    {
        $this->type = 'ldap';
        $this->params = $params;

        $this->server = $params['server'];
        $this->root = $params['root'];
        if (!empty($params['port'])) $this->port = $params['port'];
        
        if (!($this->ds = @ldap_connect($this->server, $this->port))) {
            $this->errno = 1;
            $this->errstr = 'Connection failure';
            return false;
        }

        if (!empty($params['encoding'])) {
            $this->encoding = $params['encoding'];
        }
        
        if (isset($params['bind_dn']) &&
            isset($params['bind_password'])) {
            if (!@ldap_bind($this->ds, $params['bind_dn'], $params['bind_password'])) {
                $this->errno = ldap_errno($this->ds);
                $this->errstr = ldap_error($this->ds);
                return false;
            }
        } else if (!(@ldap_bind($this->ds))) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            return false;
        }
    }
    
    /**
     * Searches the LDAP directory with the given criteria and returns
     * a filtered list of results. If no criteria are specified, all
     * records are returned.
     *
     * @param $criteria      Array containing the search criteria.
     * @param $fields        List of fields to return.
     * @param $strict_fields Array of fields which must be matched exactly.
     * @param const $match   Do an 'and' or an 'or' search (defaults to TURBA_SEARCH_AND).
     *
     * @return              Hash containing the search results.
     */
    function search($criteria, $fields, $strict_fields = array(), $match = TURBA_SEARCH_AND)
    {
        if ($match === TURBA_SEARCH_AND) {
            $glue = '(&';
        } else {
            $glue = '(|';
        }
        
        if (count($criteria) > 0) {
            $filter = $glue;
            
            /* Sort into strict and non-strict fields. */
            $strict = array();
            $loose = array();
            foreach ($criteria as $key => $val) {
                if ($this->encoding == 'utf8') {
                    $val = utf8_encode($val);
                }
                if (in_array($key, $strict_fields)) {
                    $strict[$key] = $val;
                } else {
                    $loose[$key] = $val;
                }
            }
            
            /* Add strict sub-query. */
            if (count($strict) > 0) {
                $filter .= ' (&';
                foreach ($strict as $key => $val) {
                    $filter .= ' (' . $key . '=' . $val . ')';
                }
                $filter .= ' )';
            }
            
            /* Add loose sub-query. */
            if (count($loose) > 0) {
                $filter .= ' ' . $glue;
                foreach ($loose as $key => $val) {
                    $filter .= ' (' . $key . '=*' . $val . '*)';
                }
                $filter .= ' )';
            }
            
            /* Close overall query. */
            $filter .= ' )';
            
            /* Apply source-wide filters. */
            if (!empty($this->params['filter'])) {
                $filter = $glue . $filter . '(' . $this->params['filter'] . '))';
            }
        } else {
            if (!empty($this->params['filter'])) {
                $filter = $glue . '(' . $this->params['filter'] . '))';
            } else {
                $filter = '(objectclass=*)';
            }
        }
        
        /*
         * Four11 (at least) doesn't seem to return 'cn' if you don't ask
         * for 'sn' as well.  Add 'sn' implicitly.
         */
        $attr = $fields;
        if (!in_array('sn', $attr)) $attr[] = 'sn';
        
        /* Send the query to the LDAP server and fetch the matching entries. */
        if (!($res = @ldap_search($this->ds, $this->root, $filter, $attr))) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            // FIXME: raise an exception using PEAR error handling
            return array();
        }
        
        return $this->getResults($fields, $res);
    }

    /**
     * Reads the LDAP directory for a given element and returns
     * the result's fields.
     *
     * @param $criteria      Search criteria (must be 'dn').
     * @param $dn        dn of the object to read.
     * @param $fields        List of fields to return.
     *
     * @return              Hash containing the search results.
     */
    function read($criteria, $dn, $fields)
    {
        // Only DN
        if ($criteria != 'dn') {
            return array();
        }
        
        $filter = 'objectClass=*';
        
        /*
         * Four11 (at least) doesn't seem to return 'cn' if you don't ask
         * for 'sn' as well.  Add 'sn' implicitly.
         */
        $attr = $fields;
        if (!in_array('sn', $attr)) $attr[] = 'sn';
        
        $res = @ldap_read($this->ds, $dn, $filter, $attr);
        if (!$res) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            // FIXME: raise an exception using PEAR error handling
            return array();
        }
        
        return $this->getResults($fields, $res);
    }
    
    /**
     * Get some results and clean then from a result identifier.
     *
     * @param $fields   List of fields to return.
     * @param $res  Result identifier.
     *
     * @return      Hash containing the results.
     */
    function getResults($fields, $res)
    {
        if (!($entries = @ldap_get_entries($this->ds, $res))) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            // FIXME: raise an exception using PEAR error handling
            return array();
        }
        
        /* Return only the requested fields (from $fields, above). */
        $results = array();
        for ($i = 0; $i < $entries['count']; $i++) {
            $entry = $entries[$i];
            $addr = array();
            
            foreach ($fields as $field) {
                if ($field == 'dn') {
                    $addr[$field] = $entry[$field];
                } else {
                    $addr[$field] = '';
                    if (!empty($entry[$field])) {
                        for ($j = 0; $j < $entry[$field]['count']; $j++) {
                            if (!empty($addr[$field])) $addr[$field] .= ', ';
                            if ($this->encoding == 'utf8') {
                                $addr[$field] .= utf8_decode($entry[$field][$j]);
                            } else {
                                $addr[$field] .= $entry[$field][$j];
                            }
                        }
                    }
                }
            }
            
            $results[] = $addr;
        }
        
        return $results;
    }
    
    /**
     * Closes the current LDAP connection.
     *
     * @return          The result of the ldap_close() call.
     */
    function close()
    {
        return ldap_close($this->ds);
    }
    
    /**
     * Adds the specified entry to the LDAP directory.
     */
    function addObject($attributes)
    {
        if (!isset($attributes['dn'])) {
            return new PEAR_Error('Tried to add an object with no dn: [' . serialize($attributes) . '].');
        } elseif (!isset($this->params['objectclass'])) {
            return new PEAR_Error('Tried to add an object with no objectclass: [' . serialize($attributes) . '].');
        }
        
        // Take the DN out of the attributes array
        $dn = $attributes['dn'];
        unset($attributes['dn']);
        
        // Put the Objectclass into the attributes array
        $attributes['objectclass'] = $this->params['objectclass'];
        
        if (!@ldap_add($this->ds, $dn, $attributes)) {
            return new PEAR_Error('Failed to add an object: [' . ldap_errno($this->ds) . '] "' . ldap_error($this->ds) . '" (attributes: [' . serialize($attributes) . ']).');
        } else {
            return true;
        }
    }
    
    /**
     * Deletes the specified entry from the LDAP directory.
     */
    function removeObject($object_key, $object_id)
    {
        if ($object_key != 'dn') {
            // PEAR error
            return false;
        }
        
        if (!@ldap_delete($this->ds, $object_id)) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Modifies the specified entry in the LDAP directory.
     */
    function setObject($object_key, $object_id, $attributes)
    {
        if (!@ldap_modify($this->ds, $object_id, $attributes)) {
            $this->errno = ldap_errno($this->ds);
            $this->errstr = ldap_error($this->ds);
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Build a DN based on a set of attributes and what attributes
     * make a DN for the current source.
     *
     * @param array $attributes The attributes (in driver keys) of the
     *                          object being added.
     *
     * @return string  The DN for the new object.
     */
    function makeKey($attributes)
    {
        $dn = '';
        if (is_array($this->params['dn'])) {
            foreach ($this->params['dn'] as $param) {
                if (isset($attributes[$param])) {
                    $dn .= $param . '=' . $attributes[$param] . ',';
                }
            }
        }
        
        $dn .= $this->params['root'];
        return $dn;
    }
    
}
?>
