<?php
// $Horde: turba/lib/List.php,v 1.13 2001/07/03 17:17:16 chuck Exp $

/**
 * The Turba_List:: class provides an interface for dealing with a
 * list of Turba_AbstractObjects.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.13 $
 * @since   Turba 0.0.1
 * @package turba
 */
class Turba_List {
    
    /** The array containing the Turba_Objects represented in this
        list. */
    var $objects = array();
    
    
    /**
     * Constructs a new Turba_List object.
     */
    function Turba_List() {}
    
    
    /**
     * Inserts a new list.
     *
     * @param $object         The object to insert.
     */
    function insert($object)
    {
        $this->objects[] = $object;
    }
    
    /**
     * Merges an existing Turba_List into this one.
     *
     * @param $list         The list to merge.
     */
    function merge($list)
    {
        $this->objects = array_merge($this->objects, $list);
    }
    
    /**
     * Reset our internal pointer to the beginning of the list. Use
     * this to hide the internal storage (array, list, etc.) from
     * client objects.
     */
    function reset()
    {
        reset($this->objects);
    }
    
    /**
     * Return the next Turba_Object in the list. Use this to hide
     * internal implementation details from client objects.
     *
     * @return Turba_Object $object   The next object in the list.
     */
    function next()
    {
        list(,$tmp) = each($this->objects);
        return $tmp;
    }
    
    /**
     * Return the number of Turba_Objects that are in the list. Use
     * this to hide internal implementation details from client
     * objects.
     *
     * @return integer $count The number of objects in the list.
     */
    function count()
    {
        return count($this->objects);
    }
    
    /**
     * Filters/Sorts the list based on the specified sort routine.
     *
     * @param $sort         The sort method.
     * @param $low          The low end of the sort range.
     * @param $high         The high end of the sort range.
     */
    function sort($sort = LASTNAME, $low = null, $high = null)
    {
        if (isset($low))  $low = strtolower($low);
        if (isset($high)) $high = strtolower($high);
        $sorted_objects = array();
        
        foreach ($this->objects as $object) {
            $lastname = $this->guessLastname($object->getValue('name'));
            $object->setValue('lastname', $lastname);
            $lower = strtolower($lastname);
            if ((!isset($low)  || (substr($lower, 0, strlen($low)) >= $low)) &&
                (!isset($high) || (substr($lower, 0, strlen($high)) <= $high))) {
                $sorted_objects[] = $object;
            }
        }
        
        if ($sort == LASTNAME) {
            usort($sorted_objects, array($this, 'lastNameCmp'));
        } else if ($sort == EMAIL) {
            usort($sorted_objects, array($this, 'emailCmp'));
        } else if ($sort == ALIAS) {
            usort($sorted_objects, array($this, 'aliasCmp'));
        }
        
        $this->objects = $sorted_objects;
    }
    
    /**
     * Returns a best guess at the lastname in a string.
     *
     * @param $name     String contain the full name.
     *
     * @return          String containing the last name.
     */
    function guessLastname($name)
    {
        if (!empty($name)) {
            /* Assume that last names are always before any commas. */
            if (is_int(strpos($name, ','))) {
                $name = substr($name, 0, strpos($name, ','));
            }

            /* Take out anything in parentheses. */
            $name = preg_replace('|\(.*\)|', '', $name);
            
            /* Remove single quotes. */
            $name = str_replace('\'', '', $name);
            
            $name = trim(preg_replace('|\s+|', ' ', $name));
            $namelist = explode(' ', $name);
            $name = $namelist[($nameindex = (count($namelist) - 1))];

            while (strlen($name) < 5 &&
                   strspn($name[(strlen($name) - 1)], '.:-') &&
                   isset($namelist[($nameindex - 1)])) {
                $nameindex--;
                $name = $namelist[$nameindex];
            }
        }
        return $name;
    }
    
    /**
     * Usort helper function. Compares two Turba_AbstractObjects
     * containing last names.
     *
     * @param $a        The first Turba_AbstractObject to compare.
     * @param $b        The second Turba_AbstroctObject to compare.
     *
     * @return          Integer comparison of the two names from strcmp().
     */
    function lastNameCmp($a, $b)
    {
        $acmp = strtolower($a->getValue('lastname'));
        $bcmp = strtolower($b->getValue('lastname'));
        
        return strcmp($acmp, $bcmp);
    }
    
    /**
     * Usort helper function. Compares two Turba_AbstractObjects
     * containing email addresses.
     *
     * @param $a        The first Turba_AbstractObject to compare.
     * @param $b        The second Turba_AbstroctObject to compare.
     *
     * @return          Integer comparison of the two addresses from strcmp().
     */
    function emailCmp($a, $b)
    {
        $acmp = strtolower($a->getValue('email'));
        $bcmp = strtolower($b->getValue('email'));
        
        return strcmp($acmp, $bcmp);
    }
    
    /**
     * Usort helper function. Compares two Turba_AbstractObjects
     * containing name aliases.
     *
     * @param $a        The first Turba_AbstractObject to compare.
     * @param $b        The second Turba_AbstroctObject to compare.
     *
     * @return          Integer comparison of the two aliases from strcmp().
     */
    function aliasCmp($a, $b)
    {
        $acmp = strtolower($a->getValue('alias'));
        $bcmp = strtolower($b->getValue('alias'));
        
        return strcmp($acmp, $bcmp);
    }
    
}
?>
